﻿using AdventOfCode.Common;
using AdventOfCode.Models;
using System;
using System.Collections.Generic;
using System.Linq;

namespace AdventOfCode._2022
{
    [AOC(year: 2022, day: 3)]
    public class Day3 : AOCDay
    {
        protected override AOCResponse ExecutePartA()
        {
            var rows = GetSplitInput();
            int total = 0;
            foreach (var row in rows)
            {
                var first = row.Substring(0, row.Length / 2);
                var second = row.Substring((row.Length / 2));
                var invalidItem = first.ToCharArray().Intersect(second.ToCharArray()).First();
                total += GetPriority(invalidItem);
            }
            _response.Answer = total;
            return _response;
        }

        protected override AOCResponse ExecutePartB()
        {
            var rows = GetSplitInput();
            var elves = new List<string[]>();
            int elfSet = 0;
            for (int i = 0; i < rows.Length; i++)
            {
                if (string.IsNullOrWhiteSpace(rows[i]))
                {
                    elves.Add(rows.Skip(elfSet).Take(i - elfSet).ToArray());
                    elfSet = i + 1;
                }
                if (i == rows.Length - 1)
                {
                    elves.Add(rows.Skip(elfSet).ToArray());
                }
            }
            var highestCalories = new List<int>();
            foreach (var elf in elves)
            {
                highestCalories.Add(elf.Select(x => int.Parse(x)).Sum());
            }
            _response.Answer = highestCalories.OrderByDescending(x => x).Take(3).Sum();
            return _response;
        }

        private int GetPriority(char c)
        {
            if (Char.IsUpper(c))
            {
                return (c - 'A' + 27);
            }
            if (Char.IsLower(c))
            {
                return (c - 'a' + 1);
            }
            return 0;
        }
    }
}
